@@ -71,6 +71,7 @@ class Gyoto::Metric::KerrKS
   // Constructors - Destructor
   // -------------------------
  public: 
+  GYOTO_OBJECT;
   KerrKS(); ///< Default constructor
   virtual KerrKS* clone () const;         ///< Copy constructor
   
@@ -115,11 +116,6 @@ class Gyoto::Metric::KerrKS
   virtual void circularVelocity(double const pos[4], double vel [4],
 				double dir=1.) const ;
 
-  virtual void setParameter(std::string, std::string, std::string);
-#ifdef GYOTO_USE_XERCES
-  virtual void fillElement(FactoryMessenger *fmp);
-#endif
-
  public:
 
   void MakeCst(const double* coord, double* cst) const;
