@@ -5,7 +5,7 @@
  */
 
 /*
-    Copyright 2011-2013 Thibaut Paumard, Frederic Vincent
+    Copyright 2011-2015 Thibaut Paumard, Frederic Vincent
 
     This file is part of Gyoto.
 
@@ -29,6 +29,17 @@
 #include <iostream>
 #include <fstream>
 #include <string>
+#if defined HAVE_BOOST_ARRAY_HPP
+#include <boost/array.hpp>
+#define GYOTO_ARRAY boost::array
+#else
+template <typename T, size_t sz> class GYOTO_ARRAY {
+ private:
+  T buf[sz];
+ public:
+  T& operator[](size_t c) { throwError("toto"); return buf[c] ; }
+};
+#endif
 
 namespace Gyoto {
   class Screen;
@@ -37,6 +48,7 @@ namespace Gyoto {
 #include <GyotoDefs.h>
 #include <GyotoUtils.h>
 #include <GyotoSmartPointer.h>
+#include <GyotoObject.h>
 #include <GyotoMetric.h>
 #include <GyotoSpectrometer.h>
 
@@ -144,10 +156,17 @@ namespace Gyoto {
  * \endcode
  * The mask needs to be have the same size as the Screen itself, so
  * loading a mask also sets the resolution, and changing the
- * resolution after setting a mask also removes the mask.
+ * resolution after setting a mask also removes the mask. The content
+ * of the Mask entity is parsed by Factory::fullPath(), so it can be
+ * an absolute path, a path relative to where the XML file is stored,
+ * or relative to the current working directory if prefixed with
+ * "`pwd`/".
  *
  */
-class Gyoto::Screen : protected Gyoto::SmartPointee {
+class Gyoto::Screen
+: public Gyoto::SmartPointee,
+  public Gyoto::Object
+{
   friend class Gyoto::SmartPointer<Gyoto::Screen>;
 
  private:
@@ -172,7 +191,10 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
   double distance_; ///< Distance to the observer in m
   double dmax_; ///< Maximum distance from which the photons are launched (geometrical units) 
 
-  int anglekind_; ///< Screen angles kind (0: equatorial, 1: spherical)
+  enum anglekind_e { equatorial_angles=0, rectilinear=1, spherical_angles=2};
+  typedef int anglekind_t;
+
+  anglekind_t anglekind_; ///< Screen angles kind (0: equatorial, 1: spherical)
   
   /**
    * The angles are position angle of the line of nodes (North of
@@ -205,8 +227,15 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
    */
   double freq_obs_;
 
+  /**
+   * \brief What kind of observer are we considering? (At infinity, ZAMO...)
+   *
+   */
+  std::string observerkind_;
+
  public:
-   
+  GYOTO_OBJECT;
+
   // Constructors - Destructor
   // -------------------------
   Screen() ; ///< Default constructor
@@ -245,7 +274,7 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
    * \param dist the distance expressed in the specified unit;
    * \param unit convertible to meters
    */
-  void distance(double dist, const std::string unit);
+  void distance(double dist, const std::string &unit);
 
   /// Set inclination relative to line-of-sight
   /**
@@ -311,7 +340,8 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
    * system. Content is copied.
    */
   void setObserverPos(const double pos[4]);
-
+  void observerKind(const std::string &kind);
+  std::string observerKind() const;
   void setFourVel(const double coord[4]);
   ///< Sets the observer's 4-velocity
   void setScreen1(const double coord[4]);
@@ -371,10 +401,10 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
   void metric(SmartPointer<Metric::Generic> gg); ///< Set Screen::gg_
 
   /// Get observing date in seconds
-  double time();
+  double time() const;
 
   /// Get observing date in seconds
-  double time(const std::string &);
+  double time(const std::string &) const;
 
   /// Set observing date in specified unit
   void time(double, const std::string &);
@@ -383,10 +413,10 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
   void time(double);
 
   /// Get Screen::fov_ in radians
-  double fieldOfView();
+  double fieldOfView() const;
 
   /// Get Screen::fov_ in specified unit
-  double fieldOfView(std::string unit);
+  double fieldOfView(std::string const &unit) const;
 
   /// Set Screen::fov_ in radians
   void fieldOfView(double);
@@ -401,7 +431,7 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
   /// Get direction of the center of the field
   double alpha0() const;
   /// Get direction of the center of the field in specified unit
-  double alpha0(std::string unit);
+  double alpha0(std::string const &unit)const;
   /// Set direction of the center of the field
   void delta0(double);
   /// Set direction of the center of the field in specified unit
@@ -409,13 +439,15 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
   /// Get direction of the center of the field
   double delta0() const;
   /// Get direction of the center of the field in specified unit
-  double delta0(std::string unit);
+  double delta0(std::string const &unit)const;
 
   /// Set Screen::anglekind_
-  void setAnglekind(int);
+  void anglekind(int);
+  void anglekind(std::string const&);
+  std::string anglekind() const;
 
   /// Get Screen::npix_
-  size_t resolution();
+  size_t resolution() const;
   /// Set Screen::npix_
   void resolution(size_t);
 
@@ -428,13 +460,15 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
 
   /// Retrieve const pointer to mask_
   double const * mask() const ;
+  void maskFile(std::string const &fname);
+  std::string maskFile() const;
 # ifdef GYOTO_USE_CFITSIO
 
   /// Read mask_ from FITS file
-  void fitsReadMask(std::string fname);
+  void fitsReadMask(std::string const &fname);
 
   /// Save mask_ from FITS file
-  void fitsWriteMask(std::string fname);
+  void fitsWriteMask(std::string const &fname);
 # endif
 
   /// Whether this pixel should be ray-traced
@@ -465,6 +499,15 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
    */
   void getFourVel(double fourvel[]) const;
 
+  void fourVel(std::vector<double> const &);
+  std::vector<double> fourVel() const;
+  void screenVector1(std::vector<double> const &);
+  std::vector<double> screenVector1() const;
+  void screenVector2(std::vector<double> const &);
+  std::vector<double> screenVector2() const;
+  void screenVector3(std::vector<double> const &);
+  std::vector<double> screenVector3() const;
+
   /// Get copy of Screen::screen1_
   /**
    * \param[out] output preallocated 4-element array
@@ -543,12 +586,178 @@ class Gyoto::Screen : protected Gyoto::SmartPointee {
 
 #ifdef GYOTO_USE_XERCES
  public:
-    void fillElement(FactoryMessenger *fmp); ///< called from Factory
+  void fillProperty(Gyoto::FactoryMessenger *fmp, Property const &p) const;
+
     /// Instanciate a Screen from XML entity 
     static   SmartPointer<Screen> Subcontractor(FactoryMessenger* fmp);
 #endif
 
+    /// Enum to specify whether a coordinate set (Coord1dSet or Coord2dSet) holds pixel values or angles
+    enum CoordType_e {angle, pixel};
 
+    /// Set of 1-d coordinates: indices or angles
+    /**
+     * Acts like a container (array-like) of either size_t (pixel
+     * coordinate) or double (angle) values. This container can be
+     * iterated-through using the operator++(), derefenced using the
+     * operator*() (if containing pixel coordinates) or angle() (in
+     * containing angles).
+     */
+    class Coord1dSet {
+    public:
+      /// Whether this specifier represents angles or pixels
+      const CoordType_e kind;
+    public:
+      /// Set kind during initialization
+      Coord1dSet(CoordType_e k);
+      /// Virtual destructor
+      virtual ~Coord1dSet();
+      /// Reset specifier to point to the first value
+      virtual void begin() =0;
+      /// True if pointing to something, false if end has been reached.
+      virtual bool valid() =0;
+      /// Number of values in this container
+      virtual size_t size()=0;
+      /// Get size_t value crrently pointed to
+      virtual size_t operator*() const ;
+      /// Get double value currently pointed to
+      virtual double angle() const ;
+      /// Increment iterator (point to next value)
+      virtual Coord1dSet& operator++()=0;
+    };
+
+    /// Class to specify a set of points on the Screen
+    /**
+     * Container (array-like) holding several 2D points. Can be a 2D
+     * grid of pixel coordinates or a vector of floating-point (alpha,
+     * delta) pairs, for instance.
+     */
+    class Coord2dSet {
+    public:
+      /// Whether this set holds pixels or angle specifications
+      const CoordType_e kind;
+      /// Set kind at initialisation
+      Coord2dSet(CoordType_e k);
+      /// Virtual destructor
+      virtual ~Coord2dSet();
+      /// Increment pointer
+      virtual Coord2dSet& operator++()    =0;
+      /// Get pixel coordinates
+      virtual GYOTO_ARRAY<size_t, 2> operator*  () const;
+      /// Get angle coordinates
+      virtual GYOTO_ARRAY<double, 2> angles() const ;
+      /// Reset pointer
+      virtual void begin() =0;
+      /// Whether the end has not been passed
+      virtual bool valid() =0;
+      /// Number of positions contained
+      virtual size_t size()=0;
+    };
+
+    /// Class containing 2D-points organized in a grid
+    class Grid: public Coord2dSet {
+    protected:
+    protected:
+      /// If non-NULL, cout j each tims it is incremented.
+      const char * const prefix_;
+      Coord1dSet &iset_;
+      Coord1dSet &jset_;
+    public:
+      Grid(Coord1dSet &iset, Coord1dSet &jset, const char * const p=NULL);
+      virtual Coord2dSet& operator++();
+      virtual GYOTO_ARRAY<size_t, 2> operator*  () const;
+      virtual void begin();
+      virtual bool valid();
+      virtual size_t size();
+    };
+
+    /// Class containing arbitrary 2D-points 
+    /**
+     * ispec_ and jspec_ must be the same size.
+     */
+    class Bucket : public Coord2dSet {
+    protected:
+      Coord1dSet &alpha_;
+      Coord1dSet &delta_;
+    public:
+      Bucket(Coord1dSet &iset, Coord1dSet &jset);
+      virtual Coord2dSet& operator++();
+      virtual GYOTO_ARRAY<double, 2> angles() const;
+      virtual GYOTO_ARRAY<size_t, 2> operator*() const;
+      virtual void begin();
+      virtual bool valid();
+      virtual size_t size();
+    };
+
+    /// A dummy, empty 2D set.
+    class Empty: public Coord2dSet {
+    public:
+      Empty();
+      virtual Coord2dSet& operator++();
+      virtual void begin();
+      virtual bool valid();
+      virtual size_t size();
+    };
+
+    /// 1D coordinated specifier for a range
+    class Range : public Coord1dSet {
+    protected:
+      const size_t mi_, ma_, d_, sz_;
+      size_t cur_;
+    public:
+      /// Specify min, max and step of this range.
+      Range(size_t mi, size_t ma, size_t d);
+      void begin();
+      bool valid();
+      size_t size();
+      Coord1dSet& operator++();
+      size_t operator*() const ;
+    };
+
+    /// 1D specifier for an arbitrary pixel coordinate set.
+    class Indices : public Coord1dSet {
+    protected:
+      size_t const * const indices_;
+      size_t const sz_;
+      size_t i_;
+    public:
+      Indices (size_t const*const buf, size_t sz);
+      void begin();
+      bool valid();
+      size_t size();
+      Coord1dSet& operator++();
+      size_t operator*() const ;
+    };
+
+    /// 1D specifier for an arbitrary angle coordinate set.
+    class Angles : public Coord1dSet {
+    protected:
+      double const * const buf_;
+      size_t const sz_;
+      size_t i_;
+    public:
+      Angles (double const*const buf, size_t sz);
+      void begin();
+      bool valid();
+      size_t size();
+      Coord1dSet& operator++();
+      double angle() const ;
+    };
+
+    /// 1D specifier for an angle that is repeated.
+    class RepeatAngle : public Coord1dSet {
+    protected:
+      double const val_;
+      size_t const sz_;
+      size_t i_;
+    public:
+      RepeatAngle (double val, size_t sz);
+      void begin();
+      bool valid();
+      size_t size();
+      Coord1dSet& operator++();
+      double angle() const ;
+    };
 };
 
 #endif
