@@ -1,5 +1,5 @@
 /*
-    Copyright 2011-2014 Thibaut Paumard
+    Copyright 2011-2015 Thibaut Paumard
 
     This file is part of Gyoto.
 
@@ -79,6 +79,10 @@ int yarg_Spectrum(int iarg);
 Gyoto::SmartPointer<Gyoto::Spectrometer::Generic>* yget_Spectrometer(int iarg);
 Gyoto::SmartPointer<Gyoto::Spectrometer::Generic>* ypush_Spectrometer();
 int yarg_Spectrometer(int iarg);
+void ypush_property(Gyoto::SmartPointer<Gyoto::SmartPointee>,
+		    Gyoto::Property const&, std::string, std::string);
+void yget_property(Gyoto::SmartPointer<Gyoto::SmartPointee>,
+		   Gyoto::Property const&, int, std::string, std::string);
 
 /*
   You can register your own on_eval worker. It will be called when a
@@ -133,12 +137,12 @@ void ygyoto_Spectrometer_generic_eval
 */
 // Keywords processed by ygyoto_Metric_generic_eval
 #define YGYOTO_METRIC_GENERIC_KW "prime2tdot",				\
-    "nullifycoord", "kind", "setparameter", "scalarprod",		\
+    "nullifycoord", "kind", "coordkind", "setparameter", "scalarprod",	\
     "mass", "deltamin", "deltamax", "keplerian",					\
     "unitlength", "circularvelocity",					\
     "christoffel", "xmlwrite", "clone"
 // Number of those keywords
-#define YGYOTO_METRIC_GENERIC_KW_N 14
+#define YGYOTO_METRIC_GENERIC_KW_N 15
 
 // Keywords processed by ygyoto_Astrobj_generic_eval
 #define YGYOTO_ASTROBJ_GENERIC_KW "metric", "rmax", "opticallythin",	\
@@ -250,21 +254,50 @@ void ygyoto_Spectrometer_generic_eval
 #define YGYOTO_WORKER_INIT1(BASE, CLASS, KNAMES, NKW)			\
   int rvset[1]={0}, paUsed[1]={0};					\
   *ypush_##BASE()=*YGYOTO_CAT(OBJ, _);					\
-  Gyoto::SmartPointer<CLASS> *OBJ =				\
-    (Gyoto::SmartPointer<CLASS> *)YGYOTO_CAT(OBJ, _);	\
+  {									\
+  long kidx;								\
+  Gyoto::Property const * prop;						\
+  std::string pname="", unit="";					\
+  bool unit_found, prop_pushed=false;					\
+  ++argc;								\
+  while ( (  argc > 0                 ) &&				\
+	  ( (kidx=yarg_key(argc-1)) >=0 ) &&				\
+	  ( (prop=(*YGYOTO_CAT(OBJ, _))					\
+	     ->property(pname=yfind_name(kidx)))) ) {			\
+    if ( (kidx=yarg_key(argc-3)) >=0 &&					\
+	 !strcmp(yfind_name(kidx),"unit") ) {				\
+      unit=ygets_q(argc-4);						\
+      unit_found=true;							\
+    } else {								\
+      unit_found=false;							\
+      unit="";								\
+    }									\
+    if (yarg_nil(argc-2)) {						\
+      if (prop_pushed++) y_error("Can push only one return value");	\
+      yarg_drop(1);							\
+      ypush_property(*YGYOTO_CAT(OBJ, _), *prop, pname, unit);	\
+    }									\
+    else yget_property(*YGYOTO_CAT(OBJ, _), *prop, argc-2, pname, unit); \
+    argc -= unit_found?4:2;						\
+  }									\
+  if (prop_pushed) ++*rvset;						\
+  --argc;								\
+  }									\
+  Gyoto::SmartPointer<CLASS> *OBJ =					\
+    (Gyoto::SmartPointer<CLASS> *)YGYOTO_CAT(OBJ, _);			\
   static long kglobs[NKW+1];						\
   int kiargs[NKW];							\
-  int piargs[]={-1,-1,-1,-1};						\
+  int piargs[]={-1,-1,-1,-1,-1};					\
   yarg_kw_init(const_cast<char**>(KNAMES), kglobs, kiargs);		\
   int iarg=argc, parg=0;						\
   while (iarg>=1) {							\
     iarg = yarg_kw(iarg, kglobs, kiargs);				\
     if (iarg>=1) {							\
-      if (parg<4) piargs[parg++]=iarg--;				\
-      else y_error( #CLASS " worker takes at most 4 positional arguments"); \
+      if (parg<5) piargs[parg++]=iarg--;				\
+      else y_error( #CLASS " worker takes at most 5 positional arguments"); \
     }									\
   }									\
-  GYOTO_DEBUG_ARRAY(piargs, 4);						\
+  GYOTO_DEBUG_ARRAY(piargs, 5);						\
   GYOTO_DEBUG_ARRAY(kiargs, NKW);					\
   int k=-1;								\
   char const * rmsg="Cannot set return value more than once";		\
@@ -373,31 +406,32 @@ void ygyoto_Spectrometer_generic_eval
 	(*OBJ) -> MEMBER (ygets_l(iarg));		   \
     }
 
-#define YGYOTO_WORKER_GETSET_OBJECT(MEMBER)			   \
+#define YGYOTO_WORKER_GETSET_OBJECT0(SETMEMBER,GETMEMBER,YMEMBER)   \
   if ((iarg=kiargs[++k])>=0) {					   \
     iarg+=*rvset;						   \
     if (yarg_nil(iarg)) {					   \
-      GYOTO_DEBUG << "pushing " #MEMBER << std::endl;		   \
+      GYOTO_DEBUG << "pushing " #YMEMBER << std::endl;		   \
       if ((*rvset)++) y_error("Only one return value possible");   \
-      *ypush_##MEMBER () = (*OBJ) -> get##MEMBER ();		   \
+      void * tmp = (*OBJ) -> GETMEMBER ();			   \
+      if (tmp)							   \
+	*ypush_##YMEMBER () = (*OBJ) -> GETMEMBER ();		   \
+      else ypush_long(0);					   \
     } else {							   \
-      GYOTO_DEBUG << "setting " #MEMBER << std::endl;		   \
-      (*OBJ) -> set##MEMBER (*yget_##MEMBER (kiargs[k]));	   \
+      GYOTO_DEBUG << "setting " #YMEMBER << std::endl;		   \
+      if (yarg_number(iarg)) {					   \
+        if (ygets_l(iarg) != 0)					   \
+	  y_error("Argument should be a " #YMEMBER " or 0");	   \
+	(*OBJ) -> SETMEMBER (0);				   \
+      } else							   \
+	(*OBJ) -> SETMEMBER (*yget_##YMEMBER (kiargs[k]));	   \
     }								   \
   }
 
-#define YGYOTO_WORKER_GETSET_OBJECT2(MEMBER,YMEMBER)			   \
-  if ((iarg=kiargs[++k])>=0) {					   \
-    iarg+=*rvset;						   \
-    if (yarg_nil(iarg)) {					   \
-      GYOTO_DEBUG << "pushing " #MEMBER << std::endl;		   \
-      if ((*rvset)++) y_error("Only one return value possible");   \
-      *ypush_##YMEMBER () = (*OBJ) -> MEMBER ();		   \
-    } else {							   \
-      GYOTO_DEBUG << "setting " #MEMBER << std::endl;		   \
-      (*OBJ) -> MEMBER (*yget_##YMEMBER (kiargs[k]));	   \
-    }								   \
-  }
+#define YGYOTO_WORKER_GETSET_OBJECT(MEMBER)			   \
+  YGYOTO_WORKER_GETSET_OBJECT0(set##MEMBER,get##MEMBER,MEMBER)
+
+#define YGYOTO_WORKER_GETSET_OBJECT2(MEMBER,YMEMBER)		   \
+  YGYOTO_WORKER_GETSET_OBJECT0(MEMBER,MEMBER,YMEMBER)
 
 #define YGYOTO_WORKER_SET_UNIT		 \
   if ((iarg=kiargs[++k])>=0) {		 \
@@ -459,6 +493,14 @@ void ygyoto_Spectrometer_generic_eval
 
 typedef Gyoto::SmartPointer<Gyoto::Metric::Generic> *ygyoto_yget_Metric_t(int);
 typedef Gyoto::SmartPointer<Gyoto::Metric::Generic> *ygyoto_ypush_Metric_t();
+
+typedef void ygyoto_ypush_property_t
+(Gyoto::SmartPointer<Gyoto::SmartPointee>,
+ Gyoto::Property const&, std::string, std::string);
+typedef void ygyoto_yget_property_t
+(Gyoto::SmartPointer<Gyoto::SmartPointee>,
+ Gyoto::Property const&, int, std::string, std::string);
+
 typedef int yarg_OBJTYPE_t(int);
 typedef void ygyoto_Metric_register_t(char const * const, ygyoto_Metric_eval_worker_t*);
 typedef void ygyoto_Metric_generic_eval_t(Gyoto::SmartPointer<Gyoto::Metric::Generic>*, \
@@ -538,6 +580,9 @@ typedef struct YGyotoSupplier {
   yarg_OBJTYPE_t        *yarg_Spectrometer;
   ygyoto_Spectrometer_register_t *ygyoto_Spectrometer_register;
   ygyoto_Spectrometer_generic_eval_t* ygyoto_Spectrometer_generic_eval;
+  // Properties
+  ygyoto_ypush_property_t *ypush_property;
+  ygyoto_yget_property_t *yget_property;
 } YGyotoSupplier_t;
 
 
@@ -548,6 +593,10 @@ extern YGyotoSupplier_t* YGYOTO_LOCAL_SUPPLIER;
 
 #define yget_Metric(iarg) YGYOTO_LOCAL_SUPPLIER -> yget_Metric(iarg)
 #define ypush_Metric()   YGYOTO_LOCAL_SUPPLIER  -> ypush_Metric()
+#define yget_property(a, b, c, d, e) \
+  YGYOTO_LOCAL_SUPPLIER -> yget_property(a, b, c, d, e)
+#define ypush_property(a, b, c, d)\
+  YGYOTO_LOCAL_SUPPLIER  -> ypush_property(a, b, c, d)
 #define yarg_Metric(iarg) YGYOTO_LOCAL_SUPPLIER -> yarg_Metric(iarg)
 #define ygyoto_Metric_register(kind, on_eval) \
                           YGYOTO_LOCAL_SUPPLIER -> \
